﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Web;
using Ektron.Cms.Search;
using Ektron.Cms.Search.Expressions;
using Ektron.Cms.Search.Solr;

#region About this tutorial

// This page represents a fully functional tutorial for Ektron's Search 
// Framework API. It's a sample implementation of a typical site search 
// page and provides an overview of:
//      - Crafting queries
//      - Working with results
//      - Faceted search
//      - Paging
//      - etc

// The tutorial is organized in such a way that it can be followed from
// top to bottom. For that reason, you'll find all of the necessary search
// business logic and data models captured in this source file. As you
// build your own search page, you will likely want to organize your
// code such that the appropriate abstractions exist between your
// business logic and your view.

// Comment regions can be found throughout this tutorial. These regions
// include:
//      Help        Explanations of search features and behaviors
//      Tip         Implementation tips and best practices
//      Example     Descriptions of the example code in this tutorial
//      Try it out  Opportunities to play with the code
//      Note        Caveats, support notes, etc.

// Collapse these regions as you work through the tutorial to get a
// cleaner look at the code. Another copy of this source file, with
// these regions removed, accompanies the tutorial: 
// SearchSample.aspx.cs.nocomments. If you prefer to work through the 
// code without all of the additional text, feel free to use it.

// This tutorial is targeted toward developers working with the
// Apache Solr search engine. A majority of the concepts demonstrated
// in this tutorial are applicable to all of the search engines 
// supported by Ektron. Areas where support is more limited or a feature
// is not applicable have been noted.

#endregion

#region Note: Usage information

// This tutorial may serve as a good starting point for the implementation
// of your own search page. Please keep in mind, however, that this code is 
// a sample and was not implemented for deployment in a production scenario.

// The tutorial code is provided as-is and should be thoroughly tested if
// reused in any manner other.

#endregion

public partial class SearchSample : System.Web.UI.Page
{
    protected void Page_Load(object sender, EventArgs e)
    {
        if (!IsPostBack)
        {
            #region Tip: Capture your search parameters on the query string.
            
            // By capturing your search parameters on the query string of your page
            // you make your results "linkable". This enables a site visitor to
            // bookmark their searches or share them with others.
            
            #endregion

            QueryParams parameters = new QueryParams(Request.QueryString);

            if (parameters.HasQueryText)
            {
                #region Help: What is search criteria?

                // The Ektron Search Framework API is unique in that it's comprised of a
                // single method: Search(). The differentiator is the search criteria,
                // which allows you to define your query and what data should be included
                // in the subsequent result set.
                
                // Reference:
                // http://reference.ektron.com/developer/framework/Search/KeywordSearchCriteria/

                #endregion

                KeywordSearchCriteria criteria = new KeywordSearchCriteria();                

                #region Help: Search terms

                // The stereotypical site search implementation usually consists
                // of a page with a text box and button. What a user types into
                // that text box is a collection of terms and phrases which loosely
                // describe the content that they are looking for. It's this data
                // that is assigned to the KeywordSearchCriteria.QueryText property.

                // In addition to simple terms and phrases, users may enhance their
                // queries using Ektron's keyword query syntax, which includes a
                // light set of operators:
                // - Logical operators ("AND", "OR")
                // - Relational operators ("=", "<", "<=", ">", ">=")
                // - String contains operators (":")
                // - Negation operators ("-")

                // The terms and phrases defined here are evaluated by the search engine 
                // to calculate the relevancy (or rank) of a particular search result
                // item. By default, your results are ordered by this rank, from most 
                // relevant to least relevant.

                #endregion

                criteria.QueryText = parameters.QueryText;

                #region Help: Crafting your expression tree.
                
                // Queries can be further refined by defining an expression tree
                // using the Ektron Search Framework API's library of search
                // expressions. The library includes expressions to represent
                // individual index fields (PropertyExpression), logical
                // operators (AndExpression, OrExpression), relational operators
                // (e.g. EqualsExpression, LessThanExpression), string operators
                // (e.g. ContainsExpression), and so on.

                // The ExpressionTree serves to filter your result set. It is
                // ANDed with the search terms identified by your criteria's
                // QueryText property. When the final interpretation of your
                // criteria is performed, your complete query is essentially:
                //      <query-text> AND <expression-tree>

                // Typically the ExpressionTree serves only as a filter and does
                // not impact the ranking (calculated relevance) of your individual
                // search result items. (Note: The calculation of relevance varies
                // from search engine to search engine.)

                // Reference:
                // All supported expressions are documented, with usage examples,
                // in the Ektron Developer Reference: http://reference.ektron.com                

                #endregion

                #region Help: What are property expressions?

                // The PropertyExpression class (and its various derived 
                // classes) exist to abstract away the resolution of index field names. 
                // Index field names may vary from search provider to search provider
                // and from one version of Ektron to another. 
                
                // Referencing these fields via PropertyExpressions serves to decouple 
                // your query from the underlying index schema as well as maximize 
                // portability and version compatibility.

                // PropertyExpressions are used in conjuction with other expressions
                // to define your expression tree.

                // The Ektron Search Framework API provides several classes to give you
                // easy access core Ektron properties:
                // SearchContentProperty    http://reference.ektron.com/developer/framework/Search/SearchContentProperty/
                // SearchUserProperty       http://reference.ektron.com/developer/framework/Search/SearchUserProperty/
                // SearchGroupProperty      http://reference.ektron.com/developer/framework/Search/SearchGroupProperty/
                // SearchECommerceProperty  http://reference.ektron.com/developer/framework/Search/SearchECommerceProperty/
                
                // PropertyExpressions to represent searchable content metadata definitions
                // can be constructed using the following factory class:
                //  SearchMetadataProperty   http://reference.ektron.com/developer/framework/Search/SearchMetadataProperty/

                // PropertyExpressions to represent indexed Smart Form fields can be
                // constructed using the following factory class:
                //  SearchSmartFormProperty  http://reference.ektron.com/developer/framework/Search/SearchSmartFormProperty/

                // PropertyExpressions to represent searchable custom user properties
                // can be constructed using the following factory class:
                //  SearchCustomProperty     http://reference.ektron.com/developer/framework/Search/SearchCustomProperty/

                // In more advanced scenarios, where you have defined custom fields
                // in your index (for example, when you are indexing data external
                // to the Ektron CMS), you may construct property expressions
                // using the factory methods defined on the PropertyExpression class
                // itself:
                //  PropertyExpression       http://reference.ektron.com/developer/framework/Search/PropertyExpression/   
                

                #endregion

                #region Tip: Reuse your property expressions.

                // PropertyExpressions are used throughout your search
                // criteria (ExpressionTree, ReturnProperties, Refinement,
                // OrderBy, and so on). In addition to the core Ektron
                // properties, you will likely include a number of dynamically
                // constructed PropertyExpressions (e.g. metadata definitions,
                // Smart Form fields) over the course of your search implemenation.
                
                // For that reason, it can be helpful to consolidate your search
                // properties into a single class and reuse the singleton references
                // throughout your implementation.

                // Defined as an example of this consolidation, you'll see the 
                // SearchProperties class referenced throughout this sample. The source 
                // for this class can be found if you scroll further into this tutorial.

                #endregion

                #region Tip: Simplify your queries.

                // As a rule of thumb, consider refactoring complex queries.
                // As the complexity of a query grows performance can degrade
                // significantly. Often there are ways to simplify your query
                // without jeopardizing its intent.

                #endregion

                #region Example: ExpressionTree

                // Below you'll see an example of a simple expression tree assigned
                // to our search criteria. It is filtering our results to only those
                // with a content ID greater than 0.

                // The various PropertyExpression expression classes provide a number
                // of operator overloads and helper methods to make the construction
                // of expressions easier.

                // In this example, we've made use of the overloaded greater-than
                // operator. The same expression could have been constructed in
                // a couple of other ways:
                //      criteria.ExpressionTree = SearchProperties.Id.GreaterThan(0)                
                //      criteria.ExpressionTree = new GreaterThanExpression(SearchProperties.Id, 0)

                // More complex expressions can be constructed by joining expressions with
                // logical operations:
                //  AndExpression   http://reference.ektron.com/developer/framework/Search/AndExpression/
                //  OrExpression    http://reference.ektron.com/developer/framework/Search/OrExpression/

                // The Ektron Developer Reference (http://reference.ektron.com) contains 
                // examples for all of the expressions supported by the Search Framework
                // API.

                #endregion

                #region Try it out: Expression trees

                // Expand upon the expression below to additionally filter
                // results to only those content items with a content language
                // of 1033 (or any other language present within your site.)

                // Refresh the page and issue a new query to see that filter
                // in action.

                #endregion

                criteria.ExpressionTree = SearchProperties.Id > 0;

                #region Help: Defining your result set.

                // You identify the desired columns of your search result set through
                // what are called "return properties". This is a collection of
                // PropertyExpressions representing the individual index fields that 
                // should be returned for each result item.

                #endregion

                criteria.ReturnProperties.Add(SearchProperties.Title);
                criteria.ReturnProperties.Add(SearchProperties.QuickLink);
                criteria.ReturnProperties.Add(SearchProperties.Summary);
                criteria.ReturnProperties.Add(SearchProperties.Id);
                criteria.ReturnProperties.Add(SearchProperties.DateModified);

                #region Help: What and how much?

                // Any given query has the potential to return an enormous number of results. It
                // would be very hard and extremely inefficient to work with a complete set of 
                // results. 
                
                // For that reason, we can tell the search engine to group our results
                // into pages. We can define the size of an individual page of results
                // and request which specific page in the result set we would like to see.
                
                // For example, with a page size of 10 results, we can retrieve results
                // 21 to 30 be requesting page #3.
                //  [Page: 1] 1-10
                //  [Page: 2] 11-20
                //  [Page: 3] 21-30 <--
                //  [Page: 4] 31-40
                //  ...

                // Define PagingInfo to identify exactly how many results should be 
                // returned for your query.

                #endregion

                #region Tip: Limit your page size.

                // Limit your page size to 10-20 results for optimal query performance.

                #endregion

                criteria.PagingInfo = new Ektron.Cms.PagingInfo(parameters.PageSize, parameters.Page);

                #region Help: Language processing

                // To enhance the searchability of items in the index, a search engine can
                // apply special language processing to both the query and the content as 
                // it attempts to find matches.

                // By setting the KeywordSearchCriteria.Locale property, you are identifying
                // the type of language processing that should be applied to your search terms.
                
                // If no locale is specified, the query will be processed according to the rules
                // of the default site language.

                // Note: This is not a filter. It is simply an indication of the type of language
                // processing that will be applied to the search terms. Content associated with
                // other languages may still be returned if a match is found.

                #endregion

                if (parameters.LanguageId > 0)
                {
                    criteria.Locale = Locale.Create(parameters.LanguageId);
                }

                #region Help: Requesting facets

                // Search results can be classified by the values of an individual searchable
                // property. This is called faceting (also called refinements or navigators). 
                // To receive facets with your search response, you must indicate which properties 
                // the search engine should use to derive its classifications. To do this we add 
                // a refinement specification to our search criteria.

                // A refinement specification indicates the property that we'd like to use to
                // group our data and, in some cases, additional information on how that
                // property data should be grouped.

                // Faceting is supported for string, numeric, and date fields. (Boolean faceting
                // is not supported.) When requesting facets, use the corresponding refinement
                // specification class:

                //  StringRefinementSpecification   http://reference.ektron.com/developer/framework/Search/StringRefinementSpecification/
                //  IntegerRefinementSpecification  http://reference.ektron.com/developer/framework/Search/IntegerRefinementSpecification/
                //  DecimalRefinementSpecification  http://reference.ektron.com/developer/framework/Search/DecimalRefinementSpecification/
                //  DateRefinementSpecification     http://reference.ektron.com/developer/framework/Search/DateRefinementSpecification/

                #endregion

                #region Note: Support for faceting

                // Ektron's faceting support is available with the Apache Solr and Microsoft
                // FAST Search for SharePoint 2010 search providers.

                // With Apache Solr, any searchable Ektron property may be used to request
                // facets.

                // With Microsoft FAST Search for SharePoint 2010, any searchable Ektron property
                // may be used to request facets but its corresponding "Managed Property" must
                // first be manually configured within SharePoint as a refiner.

                #endregion

                #region Tip: Limit the number of facets requested

                // Limit the number of facets on your result page. Six or fewer is a good 
                // rule-of-thumb. Anything beyond that can be confusing and overwhelming, which 
                // undermines your goal of creating a good search experience. Additionally, faceting
                // places a significant strain on your search engine. For optimal query performance,
                // it is good to apply faceting sparingly.

                #endregion

                criteria.Refinement.IsEnabled = true;                

                criteria.Refinement.Add(new StringRefinementSpecification(SearchProperties.Tags));
                criteria.Refinement.Add(new StringRefinementSpecification(SearchProperties.FolderName));
                criteria.Refinement.Add(new StringRefinementSpecification(SearchProperties.Author));
                criteria.Refinement.Add(new IntegerRefinementSpecification(SearchProperties.ContentLanguage));
                criteria.Refinement.Add(new StringRefinementSpecification(SearchProperties.Taxonomy));

                #region Example: Defining facet thresholds

                // In Ektron's integration with Apache Solr, numeric and date faceting can
                // be enhanced by specifying how you would like your data to be grouped. This is
                // accomplished by defining a series of "thresholds", which indicate the bounds
                // of each grouping. (These groupings are refered to as "facet buckets" in the
                // search response. This is covered in additional detail later in the tutorial.)

                // Below is a refinement specification to request that a facet for the publication 
                // date of our content be returned with our search response. If we were to request 
                // this facet without any additional grouping information, our results would be 
                // classified by each unique publication timestamp. That generally does not make 
                // much sense and would lead to a very confusing experience for the end user.

                // Each threshold is comprised of an upper and lower bound, defining the grouping.
                // The specification below is requesting that our facet groups data into three
                // buckets:
                //      All content published up through 12/31/2011
                //      Content published between 1/1/2012 and 12/31/2012
                //      All content from 1/1/2013 and beyond

                // The DateTime.MinValue value is used to represent the earliest possible date. This
                // can be thought of as requesting any content up to the specified upper bound.
                
                // The DateTime.MaxValue value is used to represent the latest possible date. This
                // can be thought of as requesting any content from the lower bound on up.

                // The MinValue and MaxValue behavior can be applied in thresholds for the
                // IntegerRefinementSpecification and DecimalRefinementSpecification as well.

                // Note: Thresholds are not supported in Ektron's integration with Microsoft FAST
                // Search for SharePoint 2010. Thresholds will be automatically generated by the
                // search engine, where applicable.

                #endregion

                #region Try it out: Thresholds

                // Modify the collection of thresholds, defined for the refinement specification
                // displayed below, to:
                //  - Use date ranges that are more appropriate for your content
                //  - Add additional ranges

                // Watch how these changes are reflected in the facets displayed in the sidebar
                // when your page is refreshed.

                #endregion

                DateRefinementSpecification publicationDateSpec = new DateRefinementSpecification(SearchProperties.DateModified)
                {
                    Thresholds = new List<BoundedValue<DateTime>>()
                    {
                        new BoundedValue<DateTime>(new DateTime(2011, 12, 31), DateTime.MinValue),
                        new BoundedValue<DateTime>(new DateTime(2012, 12, 31), new DateTime(2012, 1, 1)),
                        new BoundedValue<DateTime>(DateTime.MaxValue, new DateTime(2013, 1, 1))
                    }
                };                
                
                criteria.Refinement.Add(publicationDateSpec);

                #region Help: Applying refinements

                // When a user choses to refine their search results, typically acheived by
                // clicking on a link representing a facet buckets, our search criteria 
                // must be updated to reflect that selection.

                // Each facet bucket carries data identifying a filter which can be used to
                // refine your results. This data is called a refinement. When a user opts to
                // refine their query, the refinement associated with the selected facet bucket
                // can be applied to your search criteria. The search engine will interpret your
                // query in conjunction with this refinement and return you the updated result
                // set.

                #endregion

                #region Try it out: Refinements

                // Set a breakpoint on the line below and issue a query. Select one of
                // the facet buckets that are rendered on the page (for example, an 
                // author).
                
                // Watch how the criteria is updated and a subsequent query is issued.

                #endregion

                parameters.Refinements.ForEach(r => criteria.Refinement.Add(r));

                #region Help: Submitting your query

                // After you've constructed your search criteria, submit it
                // to the search manager to retrieve your results.

                #endregion

                SearchManager searchManager = new SearchManager();

                #region Help: The search response

                // All queries submitted to the SearchManager return a SearchResponseData
                // object. This object is comprised of your search results, paging info,
                // facets, and other information returned by the search engine.

                #endregion
                
                SearchResponseData response = searchManager.Search(criteria);

                txtKeywords.Text = parameters.QueryText;

                #region See: GetResults()

                // For organizational purposes, the processing of search results has
                // been captured in its own method. For details on working with these
                // results, jump down to the GetResults() method definition.

                #endregion

                rptResults.DataSource = this.GetResults(response);
                rptResults.DataBind();

                #region See: GetPaging()

                // For organizational purposes, the processing of the paging information
                // associated with our search results has been captured in its own method.
                // For details on working with paging data, jump down to the GetPaging()
                // method definition.

                #endregion

                rptPagination.DataSource = this.GetPaging(response); ;
                rptPagination.DataBind();

                #region See: GetFacet()

                // For organizational purposes, the processing of facets has been captured
                // in its own method (and a series of overloads). For details on working with 
                // facets, jump down to the appropriate GetFacet() method definition.

                #endregion

                #region Example: Rendering indexed data

                // The data retrieved from your index is not always stored in a
                // particularly readable fashion. Often it is necessary to massage
                // the data into a form that is more readily consumable.

                // In the example below, data from two of the facets that we've
                // requested is being used as-is. Data from the other three is being 
                // tweaked to render cleaner labels.

                // Consider a facet for language. The ability to refine search results
                // to a particular content language could be a powerful tool for an end
                // user. The index identifies a content item's language with a numeric
                // code however. That code doesn't mean anything to an end user so 
                // displaying it would render this facet useless. 

                // Instead, if we convert that language ID to the name of the corresponding 
                // culture, we can display a much more meaningful label for this facet's
                // values.

                // For an example of the label transformation described above, jump down to 
                // the HandleLanguageBucket() helper method.

                #endregion

                #region Try it out: Add a facet

                // Add a refinement specification for a searchable property that
                // is relevant to your site (content property, metadata, or Smart
                // Form field). Add the resulting facet to the array below using the
                // helper methods provided.

                // Set a breakpoint on the line below to follow along as the facet 
                // data is retrieved from the response and processed for rendering 
                // on the page.

                #endregion

                rptFacets.DataSource = new FacetModel[]
                {
                    this.GetFacet(response, SearchProperties.Tags, "Tags"),
                    this.GetFacet(response, SearchProperties.FolderName, "Folder"),
                    this.GetFacet(response, SearchProperties.ContentLanguage, "Language", this.HandleLanguageBucket),
                    this.GetFacet(response, SearchProperties.Author, "Author", this.HandleAuthorBucket),
                    this.GetFacet(response, SearchProperties.DateModified, "Publication Date", this.HandleDateBucket)
                    
                };

                rptFacets.DataBind();
            }
        }
    }

    /// <summary>
    /// Handles the click event for the search button on our search page, building and 
    /// submitting a URL which captures the search terms entered by the user and redirects 
    /// back to this page.
    /// </summary>    
    protected void btnSearch_Click(object sender, EventArgs e)
    {
        Response.Redirect(this.BuildUrl(new QueryParams() { QueryText = txtKeywords.Text }));
    }

    /// <summary>
    /// Builds a URL representing a search query, given the specified parameters.
    /// </summary>    
    private string BuildUrl(QueryParams parameters)
    {
        return this.ResolveUrl(Request.AppRelativeCurrentExecutionFilePath + parameters.ToString());
    }

    /// <summary>
    /// Translates the results identified by the specified SearchResponseData into
    /// a collection of result models which can be rendered on our page.
    /// </summary>
    /// <param name="response">Response to a search query</param>
    /// <returns>Collection of result model objects intended for display</returns>
    private IEnumerable<QueryResultModel> GetResults(SearchResponseData response)
    {
        #region Help: Your search results

        // The SearchResponseData class includes a property called "Results", which
        // is a collection of SearchResultData objects. 
        
        // The SearchResultData class represents an individual search result for a 
        // given query and provides access to the values of each property listed as
        // a return property in your search criteria. It is essentially a map
        // of PropertyExpression to value.

        // Property values can be accessed using the indexers exposed by the
        // SearchResultData class: SearchResultData[PropertyExpression]
        
        #endregion

        #region Try it out: SearchResultData

        // Our return properties include 'Id' and 'PublicationDate' in addition
        // to the ones utilized in the example below. Update this code to populate
        // the corresponding 'Id' and 'Date' fields of our result model.

        // Refresh the page to see this new data rendered with each result.

        #endregion

        return from r in response.Results
               select new QueryResultModel()
               {
                   Title = r[SearchProperties.Title],
                   Link = r[SearchProperties.QuickLink],
                   Summary = r[SearchProperties.Summary]
               };
    }

    /// <summary>
    /// Returns a collection of page model objects intended for use in rendering a
    /// pager for result results.
    /// </summary>
    /// <param name="response">Response to a search query</param>
    /// <returns>Collection of page model objects intended for display</returns>
    private List<PageModel> GetPaging(SearchResponseData response)
    {
        #region Help: Paging your results

        // The SearchResponseData object for your query includes a property called
        // "PagingInfo", which provides you with a total count of results for your
        // query, the index of the page that you're currently viewing, and the size
        // of that page.

        // You can use this data to implement a pager for your results, as is demonstrated
        // in the example below.

        #endregion

        List<PageModel> pages = new List<PageModel>();

        if (response.Results.Count > 0)
        {
            int startPage = Math.Max(1, response.PagingInfo.CurrentPage - 4);
            int lastPage = Math.Min(response.PagingInfo.TotalPages, response.PagingInfo.CurrentPage + 4);

            pages.Add(new PageModel()
            {
                Label = "«",
                Link = this.BuildUrl(new QueryParams(Request.QueryString) { Page = response.PagingInfo.CurrentPage - 1 }),
                IsActive = false,
                IsDisabled = response.PagingInfo.CurrentPage == 1
            });

            for (int i = startPage; i <= lastPage; i++)
            {
                pages.Add(new PageModel()
                {
                    Label = i.ToString(),
                    Link = this.BuildUrl(new QueryParams(Request.QueryString) { Page = i }),
                    IsActive = i == response.PagingInfo.CurrentPage,
                    IsDisabled = false
                });
            }

            pages.Add(new PageModel()
            {
                Label = "»",
                Link = this.BuildUrl(new QueryParams(Request.QueryString) { Page = response.PagingInfo.CurrentPage + 1 }),
                IsActive = false,
                IsDisabled = response.PagingInfo.CurrentPage == response.PagingInfo.TotalPages
            });
        }

        return pages;
    }

    /// <summary>
    /// Translates a facet for the specified property expression, from the search response, to a
    /// facet model which can be rendered on the page.
    /// </summary>
    /// <param name="response">Response to a search query</param>
    /// <param name="property">Property expression identifying the desired facet</param>
    /// <param name="header">Textual header to render with the facet</param>
    /// <returns>Facet model intended for display</returns>
    private FacetModel GetFacet(SearchResponseData response, StringPropertyExpression property, string header)
    {
        return this.GetFacet(response, property, header, null);
    }

    /// <summary>
    /// Translates a facet for the specified property expression, from the search response, to a
    /// facet model which can be rendered on the page.
    /// </summary>
    /// <param name="response">Response to a search query</param>
    /// <param name="property">Property expression identifying the desired facet</param>
    /// <param name="header">Textual header to render with the facet</param>
    /// <param name="bucketHandler">Delegate for custom handling of individual facet buckets</param>
    /// <returns>Facet model intended for display</returns>
    private FacetModel GetFacet(SearchResponseData response, StringPropertyExpression property, string header, StringFacetBucketHandler bucketHandler)
    {
        if (!response.Facets.HasFacet(property))
        {
            throw new ArgumentException(
                "The specified property is not associated with any of the facets requested " +
                "in your search criteria. If you would like to request a facet for this property, " +
                "add a corresponding StringRefinementSpecification to your search criteria."
            );
        }

        FacetModel facet = new FacetModel(header);

        #region Help: Retrieving facets

        // The SearchResponseData class exposes a property called "Facets", which
        // provides access to all facets requested via your search criteria.

        // Facets are retrieved from your search response in a manner similar to
        // how property values are retrieved from your search result data. A facet
        // is accessed via an indexer, using the appropriate PropertyExpression as
        // a key: SearchResponseData.Facets[PropertyExpression]. 
        
        // In the example below, the StringPropertyExpression passed to this method 
        // is used as our key.

        #endregion

        foreach (UniqueFacetBucket<string> bucket in response.Facets[property].Buckets)
        {
            #region Help: What is a facet bucket?

            // Each facet is broken down into a series of groupings, refered to in the
            // search API as facet buckets. An individual facet bucket is comprised of 
            // a value (often used as a label when rendering the bucket as an option
            // on your page), a count of the result items that share that value, and
            // refinement data defining a filter which can be applied to the criteria
            // of a subsequent query.            

            // Facets for string-based properties return buckets classified by unique
            // string values. All results sharing a particular unique string value
            // for the target property are grouped into the bucket for that value.

            // Facets for numeric and date properties are represented by ranges of
            // values. All results with values that fall into a particular range
            // are grouped into the corresponding bucket. 
            
            // See the GetFacet() overloads further down in this tutorial for further 
            // information.

            #endregion

            FacetItemModel item = new FacetItemModel()
            {
                Count = bucket.Count
            };

            if (bucketHandler == null)
            {
                item.Label = bucket.Value;
            }
            else
            {
                bucketHandler(bucket, item);
            }

            #region Example: Refining your results

            // Capture the refinement data, provided via the facet bucket's
            // "Refinement" property, to apply to subsequent refined queries.
            
            // In the example below, the model we generate for each facet bucket
            // has a "Link" property. We populate this property with a URL 
            // representing the current query criteria (already in our query string)
            // and add on the refinement data which defines the filter represented
            // by the individual facet bucket.

            // So, for example, when the user clicks on the facet bucket for an author
            // named "John Doe" they've essentially re-issued their original query
            // with an additional filter for only John's content.

            #endregion

            QueryParams parameters = new QueryParams(Request.QueryString)
            {
                Page = 1
            };

            parameters.Refinements.Add(bucket.Refinement);

            item.Link = this.BuildUrl(parameters);            

            facet.Items.Add(item);
        }

        return facet;
    }

    /// <summary>
    /// Translates a facet for the specified property expression, from the search response, to a
    /// facet model which can be rendered on the page.
    /// </summary>
    /// <param name="response">Response to a search query</param>
    /// <param name="property">Property expression identifying the desired facet</param>
    /// <param name="header">Textual header to render with the facet</param>    
    /// <returns>Facet model intended for display</returns>
    private FacetModel GetFacet(SearchResponseData response, DatePropertyExpression property, string header)
    {
        return this.GetFacet(response, property, header, null);
    }

    /// <summary>
    /// Translates a facet for the specified property expression, from the search response, to a
    /// facet model which can be rendered on the page.
    /// </summary>
    /// <param name="response">Response to a search query</param>
    /// <param name="property">Property expression identifying the desired facet</param>
    /// <param name="header">Textual header to render with the facet</param>
    /// <param name="bucketHandler">Delegate for custom handling of individual facet buckets</param>
    /// <returns>Facet model intended for display</returns>
    private FacetModel GetFacet(SearchResponseData response, DatePropertyExpression property, string header, DateFacetBucketHandler bucketHandler)
    {
        if (!response.Facets.HasFacet(property))
        {
            throw new ArgumentException(
                "The specified property is not associated with any of the facets requested " +
                "in your search criteria. If you would like to request a facet for this property, " +
                "add a corresponding DateRefinementSpecification to your search criteria."
            );
        }

        FacetModel facet = new FacetModel(header);

        foreach (BoundedFacetBucket<DateTime> bucket in response.Facets[property].Buckets)
        {
            FacetItemModel item = new FacetItemModel()
            {
                Count = bucket.Count
            };

            if (bucketHandler == null)
            {
                item.Label = string.Format(
                    "{0} to {1}",
                    bucket.Value.LowerBound.ToString(),
                    bucket.Value.UpperBound.ToString());
            }
            else
            {
                bucketHandler(bucket, item);
            }

            QueryParams parameters = new QueryParams(Request.QueryString)
            {
                Page = 1
            };

            parameters.Refinements.Add(bucket.Refinement);

            item.Link = this.BuildUrl(parameters);

            facet.Items.Add(item);
        }

        return facet;
    }

    /// <summary>
    /// Translates a facet for the specified property expression, from the search response, to a
    /// facet model which can be rendered on the page.
    /// </summary>
    /// <param name="response">Response to a search query</param>
    /// <param name="property">Property expression identifying the desired facet</param>
    /// <param name="header">Textual header to render with the facet</param>    
    /// <returns>Facet model intended for display</returns>
    private FacetModel GetFacet(SearchResponseData response, IntegerPropertyExpression property, string header)
    {
        return this.GetFacet(response, property, header, null);
    }

    /// <summary>
    /// Translates a facet for the specified property expression, from the search response, to a
    /// facet model which can be rendered on the page.
    /// </summary>
    /// <param name="response">Response to a search query</param>
    /// <param name="property">Property expression identifying the desired facet</param>
    /// <param name="header">Textual header to render with the facet</param>
    /// <param name="bucketHandler">Delegate for custom handling of individual facet buckets</param>
    /// <returns>Facet model intended for display</returns>
    private FacetModel GetFacet(SearchResponseData response, IntegerPropertyExpression property, string header, IntegerFacetBucketHandler bucketHandler)
    {
        if (!response.Facets.HasFacet(property))
        {
            throw new ArgumentException(
                "The specified property is not associated with any of the facets requested " +
                "in your search criteria. If you would like to request a facet for this property, " +
                "add a corresponding DateRefinementSpecification to your search criteria."
            );
        }

        FacetModel facet = new FacetModel(header);

        foreach (BoundedFacetBucket<long> bucket in response.Facets[property].Buckets)
        {
            FacetItemModel item = new FacetItemModel()
            {
                Count = bucket.Count
            };

            if (bucketHandler == null)
            {
                item.Label = string.Format(
                    "{0} to {1}",
                    bucket.Value.LowerBound.ToString(),
                    bucket.Value.UpperBound.ToString());
            }
            else
            {
                bucketHandler(bucket, item);
            }

            QueryParams parameters = new QueryParams(Request.QueryString)
            {
                Page = 1
            };

            parameters.Refinements.Add(bucket.Refinement);

            item.Link = this.BuildUrl(parameters);

            facet.Items.Add(item);
        }

        return facet;
    }

    /// <summary>
    /// Translates a facet for the specified property expression, from the search response, to a
    /// facet model which can be rendered on the page.
    /// </summary>
    /// <param name="response">Response to a search query</param>
    /// <param name="property">Property expression identifying the desired facet</param>
    /// <param name="header">Textual header to render with the facet</param>    
    /// <returns>Facet model intended for display</returns>
    private FacetModel GetFacet(SearchResponseData response, DecimalPropertyExpression property, string header)
    {
        return this.GetFacet(response, property, header, null);
    }

    /// <summary>
    /// Translates a facet for the specified property expression, from the search response, to a
    /// facet model which can be rendered on the page.
    /// </summary>
    /// <param name="response">Response to a search query</param>
    /// <param name="property">Property expression identifying the desired facet</param>
    /// <param name="header">Textual header to render with the facet</param>
    /// <param name="bucketHandler">Delegate for custom handling of individual facet buckets</param>
    /// <returns>Facet model intended for display</returns>
    private FacetModel GetFacet(SearchResponseData response, DecimalPropertyExpression property, string header, DecimalFacetBucketHandler bucketHandler)
    {
        if (!response.Facets.HasFacet(property))
        {
            throw new ArgumentException(
                "The specified property is not associated with any of the facets requested " +
                "in your search criteria. If you would like to request a facet for this property, " +
                "add a corresponding DecimalRefinementSpecification to your search criteria."
            );
        }

        FacetModel facet = new FacetModel(header);

        foreach (BoundedFacetBucket<double> bucket in response.Facets[property].Buckets)
        {
            FacetItemModel item = new FacetItemModel()
            {
                Count = bucket.Count
            };

            if (bucketHandler == null)
            {
                item.Label = string.Format(
                    "{0} to {1}",
                    bucket.Value.LowerBound.ToString(),
                    bucket.Value.UpperBound.ToString());
            }
            else
            {
                bucketHandler(bucket, item);
            }

            QueryParams parameters = new QueryParams(Request.QueryString)
            {
                Page = 1
            };

            parameters.Refinements.Add(bucket.Refinement);

            item.Link = this.BuildUrl(parameters);

            facet.Items.Add(item);
        }

        return facet;
    }

    #region Custom Facet Bucket Handling

    /// <summary>
    /// Updates the specified FacetItemModel with a custom date label to 
    /// enhance readability.
    /// </summary>
    /// <param name="bucket">Facet bucket being</param>
    /// <param name="model">Facet item model to be customized</param>
    private void HandleDateBucket(BoundedFacetBucket<DateTime> bucket, FacetItemModel model)
    {
        StringBuilder dateLabel = new StringBuilder();        

        if (bucket.Value.LowerBound == DateTime.MinValue)
        {
            // If the lower bound represents the minimum date
            // value, render the value as "Up". As in,
            // "Up to <some-date>".

            dateLabel.Append("Up");
        }
        else
        {
            // If the lower bound represents a true date value,
            // render it as is.

            dateLabel.Append(bucket.Value.LowerBound.ToString("MMM yyyy"));
        }

        if (bucket.Value.UpperBound == DateTime.MaxValue)
        {
            // If the upper bound represents the maximum date
            // value, render the value as "and Up". As in,
            // "<some-date> and Up".

            dateLabel.Append(" and Up");
        }
        else
        {
            // If the upper nound represents a true date value,
            // render it as is.

            dateLabel.Append(" to ");
            dateLabel.Append(bucket.Value.UpperBound.ToString("MMM yyyy"));
        }

        model.Label = dateLabel.ToString();
    }

    /// <summary>
    /// Updates the specified FacetItemModel with a custom author label to 
    /// enhance readability.
    /// </summary>
    /// <param name="bucket">Facet bucket being</param>
    /// <param name="model">Facet item model to be customized</param>
    private void HandleAuthorBucket(UniqueFacetBucket<string> bucket, FacetItemModel item)
    {
        // The author property is indexed as "<last-name> <first-name>". For
        // cosmetics, lets insert a comma after the last name. 
        
        // Note: This is assuming that all last names are represented by a single term.

        int firstSpace = bucket.Value.IndexOf(' ');

        item.Label = firstSpace > 0
            ? item.Label = bucket.Value.Insert(firstSpace, ",")
            : item.Label = bucket.Value;        
    }


    /// <summary>
    /// Updates the specified FacetItemModel with a custom language label to 
    /// enhance readability.
    /// </summary>
    /// <param name="bucket">Facet bucket being</param>
    /// <param name="model">Facet item model to be customized</param>
    private void HandleLanguageBucket(BoundedFacetBucket<long> bucket, FacetItemModel item)
    {
        // Language IDs aren't particularly readable. Let's update the item's
        // label with the english name for the culture.

        try
        {
            CultureInfo culture = new CultureInfo(Convert.ToInt32(bucket.Value.UpperBound));
            item.Label = culture.EnglishName;
        }
        catch(CultureNotFoundException)
        {
            item.Label = "Unknown";
        }
    }

    #endregion

    #region Delegates

    /// <summary>
    /// A delegate representing a method providing custom handling of buckets associated with
    /// a string-based facet.
    /// </summary>
    /// <param name="bucket">Facet bucket currently being processed</param>
    /// <param name="model">Facet item model which can be customized</param>
    private delegate void StringFacetBucketHandler(UniqueFacetBucket<string> bucket, FacetItemModel model);

    /// <summary>
    /// A delegate representing a method providing custom handling of buckets associated with
    /// a date-based facet.
    /// </summary>
    /// <param name="bucket">Facet bucket currently being processed</param>
    /// <param name="model">Facet item model which can be customized</param>
    private delegate void DateFacetBucketHandler(BoundedFacetBucket<DateTime> bucket, FacetItemModel model);

    /// <summary>
    /// A delegate representing a method providing custom handling of buckets associated with
    /// a decimal-based facet.
    /// </summary>
    /// <param name="bucket">Facet bucket currently being processed</param>
    /// <param name="model">Facet item model which can be customized</param>
    private delegate void DecimalFacetBucketHandler(BoundedFacetBucket<double> bucket, FacetItemModel model);

    /// <summary>
    /// A delegate representing a method providing custom handling of buckets associated with
    /// an integer-based facet.
    /// </summary>
    /// <param name="bucket">Facet bucket currently being processed</param>
    /// <param name="model">Facet item model which can be customized</param>
    private delegate void IntegerFacetBucketHandler(BoundedFacetBucket<long> bucket, FacetItemModel model);

    #endregion
}

#region Data Models

/// <summary>
/// The QueryParams class encapsulates the raw parameters describing a
/// search request to be submitted to the search engine.
/// </summary>
public class QueryParams
{
    private const string QueryTextParam = "q";
    private const string LangParam = "lang";
    private const string PageParam = "page";
    private const string PageSizeParam = "count";
    private const string RefineParam = "refine";

    /// <summary>
    /// Constructor
    /// </summary>
    public QueryParams()
    {
        this.Page = 1;
        this.PageSize = 10;
        this.Refinements = new List<Refinement>();
    }

    /// <summary>
    /// Constructor
    /// </summary>
    /// <param name="queryString">Collection of parameters representing a request query string</param>
    public QueryParams(NameValueCollection queryString)
        : this()
    {
        this.QueryText = QueryParams.GetValue(queryString, QueryParams.QueryTextParam, string.Empty);
        this.LanguageId = QueryParams.GetValue(queryString, QueryParams.LangParam, 0);
        this.Page = QueryParams.GetValue(queryString, QueryParams.PageParam, 1);
        this.PageSize = QueryParams.GetValue(queryString, QueryParams.PageSizeParam, 10);
        this.Refinements = QueryParams.DecodeRefinements(QueryParams.GetValue(queryString, QueryParams.RefineParam, string.Empty));
    }

    /// <summary>
    /// Gets or sets the search terms for the search request.
    /// </summary>
    public string QueryText { get; set; }
    
    /// <summary>
    /// Gets or sets the language ID identifying the locale for
    /// the search request. This locale is ultimately used to
    /// determine what type of linguistics should be applied
    /// to the search terms.
    /// </summary>
    public int LanguageId { get; set; }

    /// <summary>
    /// Gets or sets the index of the search result page to
    /// be displayed.
    /// </summary>
    public int Page { get; set; }

    /// <summary>
    /// Gets or sets the count of items to be displayed on a
    /// single search result page.
    /// </summary>
    public int PageSize { get; set; }

    /// <summary>
    /// Gets a collection of refinements (selected facet buckets) 
    /// to apply to the search request.
    /// </summary>
    public List<Refinement> Refinements { get; private set; }

    /// <summary>
    /// Gets a flag indicating whether or not any search terms have
    /// been provided.
    /// </summary>
    public bool HasQueryText {
        get
        {
            return !string.IsNullOrWhiteSpace(this.QueryText);
        }
    }

    /// <summary>
    /// Returns the parameters described by an instance of this class,
    /// represented as a query string.
    /// </summary>
    /// <returns>Query string representation of an instance of this class</returns>
    public override string ToString()
    {
        StringBuilder queryStringBuilder = new StringBuilder("?");
        
        queryStringBuilder.AppendFormat(
            "{0}={1}", 
            QueryParams.QueryTextParam, 
            HttpUtility.UrlEncode(this.QueryText));

        queryStringBuilder.AppendFormat("&{0}={1}", QueryParams.PageParam, this.Page);
        queryStringBuilder.AppendFormat("&{0}={1}", QueryParams.PageSizeParam, this.PageSize);

        if (this.LanguageId > 0)
        {
            queryStringBuilder.AppendFormat("&{0}={1}", QueryParams.LangParam, this.LanguageId);
        }

        if (this.Refinements.Count > 0)
        {
            queryStringBuilder.AppendFormat(
                "&{0}={1}", 
                QueryParams.RefineParam, 
                HttpUtility.UrlEncode(QueryParams.EncodeRefinements(this.Refinements)));
        }

        return queryStringBuilder.ToString();
    }

    #region Supporting Methods

    /// <summary>
    /// Retrieves a string value from the query string.
    /// </summary>
    /// <param name="queryString">Query string collection</param>
    /// <param name="key">Query string parameter name</param>
    /// <param name="defaultValue">Value to be used if key is not found</param>
    /// <returns>Value of the requested parameter</returns>
    private static string GetValue(NameValueCollection queryString, string key, string defaultValue)
    {
        return queryString[key] ?? defaultValue;
    }

    /// <summary>
    /// Retrieves a integer value from the query string.
    /// </summary>
    /// <param name="queryString">Query string collection</param>
    /// <param name="key">Query string parameter name</param>
    /// <param name="defaultValue">Value to be used if key is not found</param>
    /// <returns>Value of the requested parameter</returns>
    private static int GetValue(NameValueCollection queryString, string key, int defaultValue)
    {
        int val;
        if (!int.TryParse(queryString[key], out val))
        {
            val = defaultValue;
        }

        return val;
    }

    /// <summary>
    /// Encodes a collection of Refinement objects as a string, which can be
    /// applied to a request query string.
    /// </summary>
    /// <param name="refinements">Collection of Refinements to encode</param>
    /// <returns>Encoded collection of Refinement objects</returns>
    private static string EncodeRefinements(List<Refinement> refinements)
    {
        return string.Join(",", from r in refinements select EncodeRefinement(r));
    }

    /// <summary>
    /// Decodes a collection of Refinement objects.
    /// </summary>
    /// <param name="refinements">Encoded Refinement objects</param>
    /// <returns>Collection of Refinement objects</returns>
    private static List<Refinement> DecodeRefinements(string refinementData)
    {
        string[] refinements = refinementData.Split(new string[] { "," }, StringSplitOptions.RemoveEmptyEntries);

        return new List<Refinement>
        (
            from r in refinements select DecodeRefinement(r)
        );
    }

    /// <summary>
    /// Encodes an individual Refinement object as a string, which can be
    /// applied to a request query string.
    /// </summary>
    /// <remarks>
    /// To capture the refinement data representing a selected facet bucket
    /// in a query string, our Refinement objects ultimately need to be
    /// converted to some sort of string. 
    /// 
    /// Some search engines achieve this using a base64 encoded string of
    /// the necessary data. A similar technique is being used here. Use
    /// whatever technique best fits your particular implementation.
    /// </remarks>
    /// <param name="refinement">Refinement to encode</param>
    /// <returns>Encoded Refinement object</returns>
    public static string EncodeRefinement(Refinement refinement)
    {
        // Concatenate the identifying attributes of the specified
        // refiner and base64 encode the data.

        string refinementString = string.Format(
            "{0},{1},{2}",
            HttpUtility.UrlEncode(refinement.Property.ValueType.ToString()),
            HttpUtility.UrlEncode(refinement.Property.Name),
            HttpUtility.UrlEncode(refinement.Data));

        return Convert.ToBase64String(Encoding.UTF8.GetBytes(refinementString));
    }

    /// <summary>
    /// Decodes an individual Refinement object.
    /// </summary>
    /// <param name="data">Encoded Refinement object</param>
    /// <returns>Decoded Refinement object</returns>
    public static Refinement DecodeRefinement(string data)
    {
        Refinement refinement = null;

        // Base64 decode the refinement data. Split it into its relevant
        // components and reassemble the refinement.

        string decodedData = Encoding.UTF8.GetString(Convert.FromBase64String(data));

        string[] splitData = decodedData.Split(
            new string[] { "," }, 
            StringSplitOptions.RemoveEmptyEntries);

        if (splitData.Length == 3)
        {
            // [0] - Value type
            // [1] - Property name
            // [2] - Refinement data            

            PropertyType type = (PropertyType)Enum.Parse(
                typeof(PropertyType), 
                HttpUtility.UrlDecode(splitData[0]));

            string propertyName = HttpUtility.UrlDecode(splitData[1]);

            PropertyExpression property;

            switch (type)
            {
                case PropertyType.Text:
                    property = PropertyExpression.CreateStringProperty(splitData[1]);
                    break;

                case PropertyType.Integer:
                    property = PropertyExpression.CreateIntegerProperty(splitData[1]);
                    break;

                case PropertyType.Decimal:
                    property = PropertyExpression.CreateDecimalProperty(splitData[1]);
                    break;

                case PropertyType.DateTime:
                    property = PropertyExpression.CreateDateProperty(splitData[1]);
                    break;
            
                default:
                    throw new ArgumentException(
                        "The refinement data provided to this method does not indicate a " +
                        "property type which is supported for refinements.");
            }

            refinement = new Refinement(property, HttpUtility.UrlDecode(splitData[2]));
        }        

        return refinement;
    }

    #endregion
}

/// <summary>
/// The SearchProperties class consolidates all of the index fields
/// referenced throughout this search implementation.
/// </summary>
public class SearchProperties
{
    /// <summary>
    /// Gets a property expression representing the content ID
    /// index field.
    /// </summary>
    public static IntegerPropertyExpression Id
    {
        get { return SearchContentProperty.Id; }
    }

    /// <summary>
    /// Gets a property expression representing the content language
    /// index field.
    /// </summary>
    public static IntegerPropertyExpression ContentLanguage
    {
        get { return SearchContentProperty.Language; }
    }

    /// <summary>
    /// Gets a property expression representing the content title
    /// index field.
    /// </summary>
    public static StringPropertyExpression Title
    {
        get { return SearchContentProperty.Title; }
    }

    /// <summary>
    /// Gets a property expression representing a dynamically
    /// generated contextual summary.
    /// </summary>
    public static StringPropertyExpression Summary
    {
        get { return SearchContentProperty.HighlightedSummary; }
    }

    /// <summary>
    /// Gets a property expression representing the content 
    /// QuickLink index field.
    /// </summary>
    public static StringPropertyExpression QuickLink
    {
        get { return SearchContentProperty.QuickLink; }
    }

    /// <summary>
    /// Gets a property expression representing the content tags
    /// index field.
    /// </summary>
    public static StringPropertyExpression Tags
    {
        get { return SearchContentProperty.Tags; }
    }

    /// <summary>
    /// Gets a property expression representing the date modified
    /// index field.
    /// </summary>
    public static DatePropertyExpression DateModified
    {
        get { return SearchContentProperty.DateModified; }
    }

    /// <summary>
    /// Gets a property expression representing the content author
    /// index field.
    /// </summary>
    public static StringPropertyExpression Author
    {
        #region Help: Tokenization

        // When data enters the index, it is tokenized and filtered to ensure
        // that it is in an optimally searchable form. This may include the 
        // removal of irrelevant characters and punctuation. Markup may be
        // stripped. Text may be reformatted, stemmed, or have other linguistic
        // transformations applied to it.

        // This tokenized form is stored alongside your content and serves as
        // metadata supporting the query process.

        // With Apache Solr, facets are generated from the tokenized form of
        // your data. This is not always ideal when you're looking to render
        // meaningful options for your users.

        // Consider a facet for a property intended to capture the US state in
        // which your content has been authored. To optimize searchability the
        // search engine might break those state names apart, normalize them
        // to lower case, etc.
        
        // A simplistic example of this tokenization might look like the following
        // for a few content items tagged with this data:

        //  "New Hampshire"      -->     new | hampshire
        //  "Massachusetts"      -->     massachusetts
        //  "New York"           -->     new | york
        //  "North Dakota"       -->     north | dakota

        // If we request a facet for this property data, our results may not be
        // what we intend:

        //  new (2)
        //  hampshire (1)
        //  massachusetts (1)
        //  york (1)
        //  north (1)
        //  dakota (1)

        // In reality we want our results classified according to the complete
        // state names. We want to work with untokenized data.

        // For that reason, all Ektron index fields have an untokenized, "exact"
        // companion field. When your refiners are returned to you in a form
        // other than what you expected, try referencing this companion field
        // instead.

        // A helper class called SearchSolrProperty has been provided to facilitate
        // access to these fields. Use the CreateExactStringProperty() method to
        // create a property expression referencing the companion index field. A
        // usage example can be found in the "get" implementation below.

        // Note: This is only relevant when your search provider is Apache Solr and
        // only impacts textual data. Tokenization does not apply to numeric and date 
        // fields.
        
        // Additional information on Solr's tokenization of index data can be found
        // on the Solr wiki: http://wiki.apache.org/solr/AnalyzersTokenizersTokenFilters

        #endregion

        #region Try it out: Playing with tokenization

        // Update the return statement below to simply return the author property
        // expression, rather than its exact companion field:
        //      return SearchContentProperty.Author;

        // Refresh the page and notice how the grouping under the "Authors" facet
        // has changed.

        #endregion

        get
        {
            return SearchSolrProperty.CreateExactStringProperty(SearchContentProperty.Author);
        }
    }

    /// <summary>
    /// Gets a property expression representing the content taxonomy
    /// index field.
    /// </summary>
    public static StringPropertyExpression Taxonomy
    {
        get { return SearchSolrProperty.CreateExactStringProperty(SearchContentProperty.TaxonomyCategory); }
    }

    /// <summary>
    /// Gets a property expression representing the content folder name
    /// index field.
    /// </summary>
    public static StringPropertyExpression FolderName
    {
        get { return SearchSolrProperty.CreateExactStringProperty(SearchContentProperty.FolderName); }
    }
}

/// <summary>
/// The QueryResultModel is a data structure intended to capture
/// the search result values to render on the page.
/// </summary>
public class QueryResultModel
{
    #region Tip: Define models for your data

    // Define a model to encapsulate the data that you intended to render on
    // your page. This simplifies the development of your view and ensures
    // that some level of abstraction exists between your view and the underlying
    // search business logic that supports it.

    #endregion

    /// <summary>
    /// Gets or sets the search result title.
    /// </summary>
    public string Title { get; set; }

    /// <summary>
    /// Gets or sets the URL targeting the search
    /// result content.
    /// </summary>
    public string Link { get; set; }

    /// <summary>
    /// Gets or sets the search result summary.
    /// </summary>
    public string Summary { get; set; }

    /// <summary>
    /// Gets or sets the last modified time for the
    /// search result content.
    /// </summary>
    public DateTime Date { get; set; }

    /// <summary>
    /// Gets or sets the content ID of the search result.
    /// </summary>
    public long Id { get; set; }
}

/// <summary>
/// The FacetModel is a data structure intended to capture
/// the facet values to render on the page.
/// </summary>
public class FacetModel
{
    /// <summary>
    /// Constructor
    /// </summary>
    /// <param name="header">Textual header</param>
    public FacetModel(string header)
    {
        this.Header = header;
        this.Items = new List<FacetItemModel>();
    }

    /// <summary>
    /// Gets or sets the textual header for this facet.
    /// </summary>
    public string Header { get; set; }

    /// <summary>
    /// Gets a flag indicating whether or not this facet has
    /// any buckets associated with it.
    /// </summary>
    public bool HasItems
    {
        get
        {
            return this.Items.Count > 0;
        }
    }

    /// <summary>
    /// Gets a collection of the facet buckets associated with this
    /// facet.
    /// </summary>
    public List<FacetItemModel> Items { get; private set; }
}

/// <summary>
/// The FacetItemModel is a data structure intended to capture
/// the values of a single facet bucket to render on the page.
/// </summary>
public class FacetItemModel
{
    /// <summary>
    /// Constructor
    /// </summary>
    public FacetItemModel()
    {

    }

    /// <summary>
    /// Gets or sets the textual label for this facet item.
    /// </summary>
    public string Label { get; set; }

    /// <summary>
    /// Gets or sets the URL supporting the refinement of the
    /// query by this facet item.
    /// </summary>
    public string Link { get; set; }

    /// <summary>
    /// Gets or sets the count of results classified by this facet item.
    /// </summary>
    public long Count { get; set; }
}

/// <summary>
/// The PageModel is a data structure intended to describe an individual
/// search result page.
/// </summary>
public class PageModel
{
    /// <summary>
    /// Constructor
    /// </summary>
    public PageModel()
    {
        this.IsActive = false;
    }

    /// <summary>
    /// Gets or sets the textual label for this result page.
    /// </summary>
    public string Label { get; set; }

    /// <summary>
    /// Gets or sets the URL to access this result page.
    /// </summary>
    public string Link { get; set; }

    /// <summary>
    /// Gets or sets a flag indicating whether or not this page
    /// represents the active (selected) search page.
    /// </summary>
    public bool IsActive { get; set; }

    /// <summary>
    /// Gets or sets a flag indicating whether or not this page
    /// can be selected at this time.
    /// </summary>
    public bool IsDisabled { get; set; }
}

#endregion